#!/usr/bin/perl

use XML::Stream qw(Tree);
use Digest::SHA1 qw(sha1_hex);
use Data::Dumper;

# first open the socket stream with the server as a component
$stream = new XML::Stream(style=>"tree");
$status = $stream->Connect(hostname=>$ARGV[0], port=>$ARGV[1], namespace=>"jabber:component:accept", myhostname=>"xdb_auth_test", timeout=>10);
die("failed to connect, make sure you use the arguments: './xdb_auth_test.pl servername port secret' ") if(!(defined($status)));

# next send the hashed secret and make sure it's ok
$stream->Send($status->{id},"<handshake>".sha1_hex($status->{id}.$ARGV[2])."</handshake>");
my ($foo,$handshake) = $stream->Process();
die("invalid secret") if($$handshake[0] eq "" || &XML::Stream::GetXMLData("value",$$handshake[0],"","") ne "");

# now start processing packets
while(($foo,$pkt) = $stream->Process())
{

  # check if the connection died
  last if($pkt == -1);

  # packets are in the XML::Parser 'tree' format, ugly but it works
  # make sure we're just doing xdb packets
  my %attr = %{$pkt->[1]->[0]};
  next if($pkt->[0] ne "xdb");
  my $type = "error"; # default to error

  # for 'get's, we return an empty namespace, flagging that they exist but a check action must be used
  if($attr{type} eq "get")
  {
    $stream->Send($status->{id},"<xdb from='xdb_auth_test' to='$attr{from}' id='$attr{id}' type='result' ns='jabber:iq:auth'><query xmlns='jabber:iq:auth'/></xdb>");
    next;
  }

  # if it's a set with a check action, we'll only allow a "test" password as a demo
  if($attr{type} eq "set" && $attr{action} eq "check")
  {
    $pass = $pkt->[1]->[2]->[2];
    print "checking pass $pass for user $attr{to}\n";
    $type = "result" if($pass eq "test");
  }

  # send back the response, be it an error or result type
  $stream->Send($status->{id},"<xdb from='xdb_auth_test' to='$attr{from}' id='$attr{id}' type='$type' ns='jabber:iq:auth'/>");
}

die("disconnected from server");

